@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2005 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -16,9 +16,7 @@
 """imports checkers for Python code
 """
 
-__revision__ = "$Id: imports.py,v 1.46 2005-11-10 17:26:27 syt Exp $"
-
-from logilab.common import get_cycles
+from logilab.common.graph import get_cycles
 from logilab.common.modutils import is_standard_module, is_relative, \
      get_module_part
 from logilab.common.ureports import VerbatimText, Paragraph
@@ -248,8 +246,12 @@ given file (report R0402 must not be disabled)'}
         basename = node.modname
         if basename == '__future__':
             # check this is the first non docstring statement in the module
-            if node.previous_sibling():
-                self.add_message('W0410', node=node)                    
+            prev = node.previous_sibling()
+            if prev:
+                # consecutive future statements are possible
+                if not(isinstance(prev, astng.From)
+                       and prev.modname == '__future__'):
+                    self.add_message('W0410', node=node)
         self._check_deprecated(node, basename)
         relative = self._check_relative(node, basename)
         for name, _ in node.names:
