@@ -16,6 +16,7 @@
 """try to find more bugs in the code using astng inference capabilities
 """
 
+from logilab.common.compat import set
 from logilab import astng
 
 from pylint.interfaces import IASTNGChecker
@@ -24,10 +25,14 @@ from pylint.checkers.utils import safe_infer, is_super, display_type
 
 MSGS = {
     'E1101': ('%s %r has no %r member',
-              'Used when a class is accessed for an unexistant member.'),
+              'Used when a variable is accessed for an unexistant member.'),
     'E1102': ('%s is not callable',
               'Used when an object being called has been infered to a non \
               callable object'),
+    'E1103': ('%s %r has no %r member (but some types could not be inferred)',
+              'Used when a variable is accessed for an unexistant member, but \
+              astng was not able to interpret all possible types of this \
+              variable.'),
     'E1111': ('Assigning to function call which doesn\'t return',
               'Used when an assigment is done on a function call but the \
               infered function doesn\'t return anything.'),
@@ -48,14 +53,21 @@ class TypeChecker(BaseChecker):
     msgs = MSGS
     priority = -1
     # configuration options
-    options = (
-               ('ignore-mixin-members',
+    options = (('ignore-mixin-members',
                 {'default' : True, 'type' : 'yn', 'metavar': '<y_or_n>',
                  'help' : 'Tells wether missing members accessed in mixin \
 class should be ignored. A mixin class is detected if its name ends with \
 "mixin" (case insensitive).'}
                 ),
-               
+
+               ('ignored-classes',
+                {'default' : ('SQLObject',),
+                 'type' : 'csv',
+                 'metavar' : '<members names>',
+                 'help' : 'List of classes names for which member attributes \
+should not be checked (useful for classes with attributes dynamicaly set).'}
+                 ),
+
                ('zope',
                 {'default' : False, 'type' : 'yn', 'metavar': '<y_or_n>',
                  'help' : 'When zope mode is activated, consider the \
@@ -89,20 +101,23 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
         except astng.InferenceError:
             return
         # list of (node, nodename) which are missing the attribute
-        missingattr = []
+        missingattr = set()
         ignoremim = self.config.ignore_mixin_members
+        inference_failure = False
         for owner in infered:
             # skip yes object
             if owner is astng.YES:
+                inference_failure = True
                 continue
-            # if there is ambiguity, skip None
-            if len(infered) > 1 and isinstance(owner, astng.Const) \
-                   and owner.value is None:
+            # skip None anyway
+            if isinstance(owner, astng.Const) and owner.value is None:
                 continue
             # XXX "super" / metaclass call
             if is_super(owner) or getattr(owner, 'type', None) == 'metaclass':
                 continue
             name = getattr(owner, 'name', 'None')
+            if name in self.config.ignored_classes:
+                continue
             if ignoremim and name[-5:].lower() == 'mixin':
                 continue
             try:
@@ -110,7 +125,7 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
             except AttributeError:
                 # XXX method / function
                 continue
-            except astng.NotFoundError:
+            except astng.NotFoundError, ex:
                 if isinstance(owner, astng.Instance) \
                        and owner.has_dynamic_getattr():
                     continue
@@ -118,15 +133,27 @@ zope\'s acquisition mecanism and so shouldn\'t trigger E0201 when accessed \
                 if owner.name == 'Values' and \
                        owner.root().name in ('optik', 'optparse'):
                     continue
-                missingattr.append((owner, name))
+                missingattr.add((owner, name))
                 continue
             # stop on the first found
             break
         else:
             # we have not found any node with the attributes, display the
             # message for infered nodes
+            done = set()
             for owner, name in missingattr:
-                self.add_message('E1101', node=node,
+                if isinstance(owner, astng.Instance):
+                    actual = owner._proxied
+                else:
+                    actual = owner
+                if actual in done:
+                    continue
+                done.add(actual)
+                if inference_failure:
+                    msgid = 'E1103'
+                else:
+                    msgid = 'E1101'
+                self.add_message(msgid, node=node,
                                  args=(display_type(owner), name,
                                        node.attrname))
 
