@@ -1,3 +1,5 @@
+# Copyright (c) 2003-2006 Sylvain Thenault (thenault@gmail.com).
+# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
@@ -10,10 +12,7 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-""" Copyright (c) 2002-2006 LOGILAB S.A. (Paris, FRANCE).
- http://www.logilab.fr/ -- mailto:contact@logilab.fr
-
-HTML reporter
+"""HTML reporter
 """
 
 __revision__ = "$Id: html.py,v 1.14 2006-03-08 15:53:41 syt Exp $"
@@ -50,7 +49,9 @@ class HTMLReporter(BaseReporter):
     def _display(self, layout):
         """launch layouts display
         
-        overriden from BaseReporter to add a blank line...
+        overridden from BaseReporter to add insert the messages section
+        (in add_message, message is not displayed, just collected so it
+        can be displayed in an html table)
         """
         if self.msgs:
             # add stored messages to the layout
