@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2007 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -57,24 +57,25 @@ def exception_str(ex):
     return 'in %s\n:: %s' % (ex.file, ', '.join(ex.args))
 
 class LintTestUsingModule(testlib.TestCase):            
-
+    package = 'input'
+    linter = linter
     def test_functionality(self):
-        tocheck = ['input.'+self.module]
+        tocheck = [self.package+'.'+self.module]
         if self.depends:
-            tocheck += ['input.%s' % name.replace('.py', '')
+            tocheck += [self.package+'.%s' % name.replace('.py', '')
                         for name, file in self.depends]
         self._test(tocheck)
         
     def _test(self, tocheck):
         if INFO_TEST_RGX.match(self.module):
-            linter.enable_message_category('I')
+            self.linter.enable_message_category('I')
         else:
-            linter.disable_message_category('I')
+            self.linter.disable_message_category('I')
         try:
-            linter.check(tocheck)
+            self.linter.check(tocheck)
         except Exception, ex:
             # need finalization to restore a correct state
-            linter.reporter.finalize()
+            self.linter.reporter.finalize()
             ex.file = tocheck
             ex.__str__ = new.instancemethod(exception_str, ex, None)
             raise
@@ -84,7 +85,7 @@ class LintTestUsingModule(testlib.TestCase):
             output = open(self.output)
             expected = output.read().strip()
             output.close()
-        got = linter.reporter.finalize().strip()
+        got = self.linter.reporter.finalize().strip()
         try:
             self.assertLinesEquals(got, expected)
         except Exception, ex:
@@ -96,26 +97,47 @@ class LintTestUsingModule(testlib.TestCase):
 class LintTestUsingFile(LintTestUsingModule):            
                 
     def test_functionality(self):
-        tocheck = ['input/' + self.module + '.py']
+        tocheck = [self.package+'/' + self.module + '.py']
         if self.depends:
-            tocheck += ['input/%s' % name for name, file in self.depends]
+            tocheck += [self.package+'/%s' % name for name, file in self.depends]
         self._test(tocheck)
 
 
-class TestTests(unittest.TestCase):
+class TestTests(testlib.TestCase):
     """check that all testable messages have been checked"""
     def test(self):
-        todo = linter._messages.keys()
-        for msg_id in test_reporter.message_ids.keys():
-            todo.remove(msg_id)
+        # skip rpython checker and fatal messages
+        todo = [msgid for msgid in linter._messages.keys() if msgid[1:3] != '12' and msgid[0] != 'F']
+        for msgid in test_reporter.message_ids.keys():
+            try:
+                todo.remove(msgid)
+            except ValueError:
+                continue
         todo.sort()
         if PY25:
-            self.assertEqual(todo, ['E0503', 'E1010', 'F0002', 'F0202', 'F0321', 'I0001'])
+            self.assertEqual(todo, ['E0503', 'E1010', 'I0001'])
         elif PY23:
-            self.assertEqual(todo, ['E0503', 'F0002', 'F0202', 'F0321', 'I0001'])
+            self.assertEqual(todo, ['E0503', 'I0001'])
         else: # python < 2.3
-            self.assertEqual(todo, ['F0002', 'F0202', 'F0321', 'I0001'])
-        
+            self.assertEqual(todo, ['I0001'])
+
+#bycat = {}
+#for msgid in linter._messages.keys():
+#    bycat[msgid[0]] = bycat.setdefault(msgid[0], 0) + 1
+#for cat, val in bycat.items():
+#    print '%s: %s' % (cat, val)
+#print 'total', sum(bycat.values())
+#
+# on 2007/02/17:
+#
+# W: 48
+# E: 42
+# R: 15
+# C: 13
+# F: 7
+# I: 5
+# total 130
+
 def make_tests(filter_rgx):
     """generate tests classes from test info
     
@@ -190,6 +212,6 @@ if __name__=='__main__':
     if len(sys.argv) > 1:            
         FILTER_RGX = sys.argv[1]
         del sys.argv[1]
-    unittest.main(defaultTest='suite')
+    testlib.unittest_main(defaultTest='suite')
 
 
