@@ -11,8 +11,8 @@ class Temelekefe:
         class Toto:
             """toto nested class"""
             def __init__(self):
-                pass
+                self.attr = 2
             def toto_method(self):
                 """toto nested class method"""
                 print self
-        print 'error ?', Toto
+        print 'error ?', self, Toto
